package net.wanji.utc.task;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.databus.dao.entity.CrossLightsStatusHistPO;
import net.wanji.databus.dao.mapper.CrossInfoMapper;
import net.wanji.databus.dao.mapper.CrossLightsStatusHistMapper;
import net.wanji.databus.dao.mapper.ManufacturerInfoMapper;
import net.wanji.databus.po.CrossInfoPO;
import net.wanji.databus.po.ManufacturerInfoPO;
import net.wanji.databus.po.SignalStatusLogPO;
import net.wanji.databus.vo.LightsStatusVO;
import net.wanji.databus.vo.LightsStatusVO2;
import net.wanji.feign.service.UtcDTFeignClients;
import net.wanji.feign.service.UtcHisenseFeignClients;
import net.wanji.utc.common.constant.Constants;
import net.wanji.utc.common.typeenum.BasicEnum;
import net.wanji.utc.kafka.ProducerHandler;
import net.wanji.utc.service.runninginfo.HkLightsStatusService;
import net.wanji.utc.service.runninginfo.SignalStatusService;
import net.wanji.utc.websocket.RealTimeDataWebSocket;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 订阅信号运行状态、告警、灯态
 *
 * @author Kent HAN
 * @date 2022/11/21 9:01
 */
@Component
@Slf4j
@SuppressWarnings("all")
public class SignalStatusTask {
    @Autowired
    SignalStatusService signalStatusService;
    @Resource(name = "commonThreadPoolExecutor")
    ThreadPoolTaskExecutor commonThreadPoolExecutor;

    @Resource
    private HkLightsStatusService hkLightsStatusService;
    @Resource
    private ManufacturerInfoMapper manufacturerInfoMapper;
    @Resource
    private CrossInfoMapper crossInfoMapper;
    @Resource
    private UtcDTFeignClients utcDTFeignClients;
    @Resource
    private CrossLightsStatusHistMapper crossLightsStatusHistMapper;
    @Autowired
    private ProducerHandler producerHandler;

    private static Map<String, List<LightsStatusVO>> lightStatusCache = new HashMap<>();
    private static final Map<String, Integer> runModeTimeMap = new ConcurrentHashMap<>();
    @Qualifier("net.wanji.feign.service.UtcHisenseFeignClients")
    @Autowired
    private UtcHisenseFeignClients utcHisenseFeignClients;

    private static final ConcurrentHashMap<String, List<LightsStatusVO>> produceListMap = new ConcurrentHashMap<>();

    // 运行状态、告警，5分钟一次
    @Scheduled(fixedRate = 1 * 60 * 1000)
    public void runningStatusAlarm() throws Exception {
        List<SignalStatusLogPO> signalStatusLogPOS = signalStatusService.runningStatusAlarm();
    }

    // 灯态，每秒一次
    @Scheduled(fixedRate = 1 * 1000)
    public void lightStatus() {
        ConcurrentHashMap<String, RealTimeDataWebSocket> evenWarnWebSocketMap = RealTimeDataWebSocket.getEvenWarnWebSocketMap();
        try {
            if (!evenWarnWebSocketMap.isEmpty()) {
                Map<String, List<Map.Entry<String, RealTimeDataWebSocket>>> crossEntryMap = getCrossEntryMap(evenWarnWebSocketMap);
                for (Map.Entry<String, List<Map.Entry<String, RealTimeDataWebSocket>>> entry : crossEntryMap.entrySet()) {
                    //commonThreadPoolExecutor.execute(() -> {
                    ObjectMapper mapper = JacksonUtils.getInstance();
                    String crossId = entry.getKey();
                    List<LightsStatusVO> lightsStatusVOS = null;
                    CrossInfoPO crossInfoPO = crossInfoMapper.selectByPrimaryKey(crossId);
                    if (Objects.isNull(crossInfoPO)) {
                        continue;
                    }
                    ManufacturerInfoPO manufacturerInfoPO = manufacturerInfoMapper.selectById(crossInfoPO.getManufacturerId());
                    try {
                        if (StringUtils.equals(BasicEnum.ManufacturerEnum.HK.getCode(), manufacturerInfoPO.getCode())) {
                            lightsStatusVOS = hkLightsStatusService.getHkLightsStatus(crossId);
                        } else if (StringUtils.equals(BasicEnum.ManufacturerEnum.DT.getCode(), manufacturerInfoPO.getCode())) {
                            JsonViewObject jsonViewObject = utcDTFeignClients.lightStatus(crossId);
                            lightsStatusVOS = mapper.convertValue(jsonViewObject.getContent(), new TypeReference<List<LightsStatusVO>>() {
                            });
                        } else if (StringUtils.endsWithIgnoreCase(BasicEnum.ManufacturerEnum.HISENSE.getCode(), manufacturerInfoPO.getCode())) {
                            JsonViewObject jsonViewObject = utcHisenseFeignClients.lightStatus(crossId);
                            lightsStatusVOS = mapper.convertValue(jsonViewObject.getContent(), new TypeReference<List<LightsStatusVO>>() {
                            });
                        } else if (StringUtils.endsWithIgnoreCase("STATIC", manufacturerInfoPO.getCode())) {
                            LightsStatusVO lightsStatusVO = SignalStatus4StaticSchemeTask.currentCrossLightStatusCache.get(crossId);
                            if (Objects.nonNull(lightsStatusVO)) {
                                lightsStatusVOS = new ArrayList<>();
                                lightsStatusVOS.add(lightsStatusVO);
                            }
                        }
                    } catch (Exception e) {
                        log.error("获取厂商灯态异常", e);
                    }

                    List<Map.Entry<String, RealTimeDataWebSocket>> value = entry.getValue();
                    if (!CollectionUtils.isEmpty(lightsStatusVOS)) {
                        for (LightsStatusVO lightsStatusVO : lightsStatusVOS) {
                            if (Objects.nonNull(lightsStatusVO) && Objects.equals(lightsStatusVO.getCrossId(), crossId)) {
                                calculateContolCountDown(crossId, lightsStatusVO);
                                try { // 相同路口不同websocket统一发送灯态
                                    // 新版本，测试，未更新
                                    //List<LightsStatusVO2> listResult = lightsStatusVOCopy(lightsStatusVOS);
                                    String json = mapper.writeValueAsString(lightsStatusVOS);
                                    for (Map.Entry<String, RealTimeDataWebSocket> socketEntry : value) {
                                        String crossIdStr = socketEntry.getKey();
                                        RealTimeDataWebSocket webSocket = socketEntry.getValue();
                                        webSocket.sendInfo(json, crossIdStr);
                                    }
                                    insertIntoHist(crossId, json);
                                    produceListMap.put(crossId, lightsStatusVOS);
                                } catch (Exception e) {
                                    log.error("RealTimeDataWebSocket发送异常，异常信息：", e);
                                }
                            }
                        }
                    }
                    //});
                }
                sendKafka();
            }
        } catch (Exception e) {
            log.error("实时推送灯态数据失败", e);
        }
    }

    private static List<LightsStatusVO2> lightsStatusVOCopy(List<LightsStatusVO> lightsStatusVOS) throws Exception{
        List<LightsStatusVO2> listResult = null;
        try {
            listResult = new ArrayList<>(lightsStatusVOS.size());
            for (LightsStatusVO lightsStatusVO : lightsStatusVOS) {
                Integer cyclePhaseCountDown = lightsStatusVO.getCyclePhaseCountDown();
                LightsStatusVO2 lightsStatusVO2 = new LightsStatusVO2();
                BeanUtils.copyProperties(lightsStatusVO, lightsStatusVO2);
                Map<String, Object> dirLampGroupMap = lightsStatusVO.getDirLampGroupMap();
                List<LightsStatusVO2.DirInfo> dirList = new ArrayList<>();
                Map<String, Object> phaseMap = lightsStatusVO.getPhaseMap();
                for (Map.Entry<String, Object> groupEntry : dirLampGroupMap.entrySet()) {
                    LightsStatusVO2.DirInfo dirInfo = new LightsStatusVO2.DirInfo();
                    String dir = groupEntry.getKey();
                    Map<Integer, String> turnColorMap = (Map<Integer, String>) groupEntry.getValue();
                    List<LightsStatusVO2.TurnInfo> turnList = new ArrayList<>();
                    for (Map.Entry<Integer, String> turnEntry : turnColorMap.entrySet()) {
                        Integer turn = turnEntry.getKey();
                        String color = turnEntry.getValue();
                        LightsStatusVO2.TurnInfo turnInfo = new LightsStatusVO2.TurnInfo();
                        turnInfo.setTurn(String.valueOf(turn));
                        turnInfo.setColor(color);
                        String key = dir.concat(Constants.SEPARATOR_UNDER_LINE).concat(String.valueOf(turn));
                        Map<Integer, Integer> turnMap = (Map<Integer, Integer>) phaseMap.get(dir);
                        if (!turnMap.isEmpty()) {
                            Integer countDown = turnMap.get(turn);
                            turnInfo.setCountDown(countDown);
                        }
                        turnList.add(turnInfo);
                    }
                    dirInfo.setDir(dir);
                    dirInfo.setTurnList(turnList);
                    dirList.add(dirInfo);
                }
                lightsStatusVO2.setDirLampGroupMapList(dirList);
                listResult.add(lightsStatusVO2);
            }
        } catch (Exception e) {
            log.error("灯态推送转换V2异常：", e);
            throw new Exception(e);
        }
        return listResult;
    }

    /**
     * 灯态数据发送kafka
     */
    private void sendKafka() {
        if (!produceListMap.isEmpty()) {
            List<LightsStatusVO> list = new ArrayList<>(produceListMap.size());
            for (Map.Entry<String, List<LightsStatusVO>> entry : produceListMap.entrySet()) {
                list.addAll(entry.getValue());
            }
            producerHandler.send("cross_lights_status", list);
        }
    }

    /**
     * 计算控制倒计时
     *
     * @param crossId
     * @param lightsStatusVO
     */
    private static void calculateContolCountDown(String crossId, LightsStatusVO lightsStatusVO) {
        Integer coutTime = 0;
        // 特殊模式控制逻辑
        if (Arrays.asList(2, 3, 4, 9).contains(Integer.valueOf(lightsStatusVO.getRunMode()))) {
            if (runModeTimeMap.isEmpty() || Objects.isNull(runModeTimeMap.get(crossId))) {
                runModeTimeMap.put(crossId, coutTime);
            } else {
                Integer countTimeCahce = runModeTimeMap.get(crossId);
                runModeTimeMap.put(crossId, ++countTimeCahce);
            }
            lightsStatusVO.setCycleCountDown(runModeTimeMap.get(crossId));
        } else if (StringUtils.equals(lightsStatusVO.getRunMode(), "11")) {
            runModeTimeMap.remove(crossId);
        }
        Constants.controlModeMap.put(crossId, lightsStatusVO.getRunMode());
    }

    private static Map<String, List<Map.Entry<String, RealTimeDataWebSocket>>> getCrossEntryMap(ConcurrentHashMap<String, RealTimeDataWebSocket> evenWarnWebSocketMap) {
        Map<String, List<Map.Entry<String, RealTimeDataWebSocket>>> tempMap = new ConcurrentHashMap<>();
        for (Map.Entry<String, RealTimeDataWebSocket> item : evenWarnWebSocketMap.entrySet()) {
            String crossId = "";
            String crossIdStr = item.getKey();
            if (StringUtils.isNotBlank(crossIdStr)) {
                String[] split = crossIdStr.split(",");
                crossId = split[0];
            }
            List<Map.Entry<String, RealTimeDataWebSocket>> socketList = new ArrayList<>();
            if (tempMap.containsKey(crossId)) {
                socketList = tempMap.get(crossId);
                socketList.add(item);
            } else {
                socketList.add(item);
            }
            tempMap.put(crossId, socketList);
        }
        return tempMap;
    }

    /**
     * 插入路口灯态历史表
     *
     * @param crossId 路口ID
     * @param json    灯态JSON字符串
     * @throws Exception 异常处理
     */
    private void insertIntoHist(String crossId, String json) throws Exception {
        try {
            CrossLightsStatusHistPO crossLightsStatusHistPO = new CrossLightsStatusHistPO();
            crossLightsStatusHistPO.setCrossId(crossId);
            LocalDateTime now = LocalDateTime.now();
            Integer batchTime = (int) now.toEpochSecond(ZoneOffset.of("+8"));
            crossLightsStatusHistPO.setBatchTime(batchTime);
            crossLightsStatusHistPO.setLightsStatusJson(json);
            crossLightsStatusHistMapper.insertOne(crossLightsStatusHistPO);
        } catch (Exception e) {
            log.error("插入路口灯态历史表异常", e);
            throw new Exception(e);
        }
    }
}
